@@ -1,4 +1,4 @@
-" Flaskbox "
+" Drink "
 from __future__ import absolute_import, with_statement
 
 # configuration time
@@ -8,6 +8,7 @@ from drink.config import config, BASE_DIR
 
 import bottle
 import os
+
 bottle.TEMPLATE_PATH.append(os.path.join(BASE_DIR,'templates'))
 STATIC_PATH = os.path.abspath(os.path.join(BASE_DIR, "static"))
 DB_PATH = config.get('server', 'database') or os.path.abspath(os.path.join(BASE_DIR, os.path.pardir, "database"))
@@ -30,7 +31,8 @@ if i >= 0:
 # http
 from bottle import route, static_file, request, response, redirect as rdr, abort
 # templating
-from bottle import jinja2_view as view, jinja2_template as template
+import functools
+template = functools.partial(bottle.template, req=request, template_adapter=bottle.Jinja2Template)
 
 # Load Basic objects
 from .objects import classes, get_object, init as init_objects
@@ -48,6 +50,35 @@ from .zdb import Database
 db = Database(bottle.app(), DB_CONFIG)
 import transaction
 
+# Declare "drink" wsgi loader
+class DrinkServer(bottle.ServerAdapter):
+    """ Drink-flavored bottle runner.
+    Will try gevent, paste, bjoern >=1.1, rocket, fapws3 or wsgiref (in this order)
+    """
+    adapters = [bottle.GeventServer, bottle.PasteServer,
+        bottle.RocketServer, bottle.FapwsServer, bottle.WSGIRefServer]
+
+    def run(self, handler):
+        adapters = list(self.adapters)
+
+#        try:
+#            import bjoern
+#        except ImportError:
+#            pass
+#        else:
+#            if bjoern.version >= (1, 1, 0):
+#                adapters.insert(1, bottle.BjoernServer)
+
+
+        for sa in adapters:
+            try:
+                print "* Trying %s"%sa.__name__
+                return sa(self.host, self.port, **self.options).run(handler)
+            except ImportError:
+                pass
+
+bottle.server_names['drink'] = DrinkServer
+
 # Real code starts here
 
 class Authenticator(object):
@@ -110,7 +141,9 @@ class Authenticator(object):
 @route('/')
 def main_index():
     request.identity = Authenticator()
-    return classes[config.get('server', 'index')](db.db).view()
+    o = classes[config.get('server', 'index')](db.db)
+    o.disable_ajax = True
+    return o.view()
 
 @route('/static/:filename#.*#')
 def server_static(filename):
@@ -142,9 +175,11 @@ def glob_index(objpath="/"):
         response.content_type = "text/plain"
         return o
     else:
-        return o.view()
+        return getattr(o, o.default_action)()
 
 def init():
+    from drink.objects.finder import reset
+    reset()
     with db as root:
         root.clear()
 
@@ -154,7 +189,7 @@ def init():
             user = None
 
             def access(self, obj):
-                return 'rw'
+                return 'rowat'
 
         request.identity = FakeId()
         groups = root['groups'] = obj.GroupList('groups', '/')
@@ -196,6 +231,14 @@ def init():
             elt = classes[ name ](pagename, '/')
             root[pagename] = elt
 
+        if 'pages' in root:
+            mdown = classes['Web page (markdown)']('help', '/pages/')
+            help = os.path.abspath(os.path.join(BASE_DIR, os.path.pardir, "HELP.md"))
+            mdown.content = open(help).read()
+            mdown.owner = admin
+            mdown.min_rights = 'r'
+            root['pages']['help'] = mdown
+
 
 def startup():
     import sys
@@ -207,12 +250,24 @@ def startup():
     else:
         setproctitle.setproctitle('drink')
 
+    # handle commands
+    # TODO: create "update" command
+    if len(sys.argv) == 2 and sys.argv[1] == "help":
+        print """
+Drink help
 
-    if len(sys.argv) == 2 and sys.argv[1] == "init":
+commands:
+    init: reset database
+    pack: pack database (more compact/faster)
+    debug: run a debugger after loading
+    help: this help :)
+
+Without any argument, it will just run the server.
+if DEBUG environment variable is set, it will start in debug mode.
+        """
+    elif len(sys.argv) == 2 and sys.argv[1] == "init":
         init()
         db.pack()
-    # TODO: rename __init_
-    # TODO: create "update" command
     elif len(sys.argv) == 2 and sys.argv[1] == "pack":
         db.pack()
     elif len(sys.argv) == 2 and sys.argv[1] == "debug":
@@ -233,11 +288,16 @@ def startup():
             if reset_required:
                 init()
 
+        for name, klass in classes.items():
+            if getattr(klass, 'hidden_class', False):
+                del classes[name]
+
+        # And now, http/wsgi part
+
         host = config.get('server', 'host')
         port = int(config.get('server', 'port'))
         mode = config.get('server', 'backend')
 
-
         # try some (optional) asynchronous optimization
 
         async = False
